@@ -20,6 +20,7 @@ package org.apache.empire.jsf2.app;
 
 import java.io.File;
 import java.io.IOException;
+import java.text.MessageFormat;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
@@ -31,6 +32,7 @@ import javax.faces.component.UIInput;
 import javax.faces.component.UIViewRoot;
 import javax.faces.context.ExternalContext;
 import javax.faces.context.FacesContext;
+import javax.faces.event.ActionEvent;
 import javax.servlet.ServletContext;
 import javax.servlet.http.HttpServletRequest;
 
@@ -86,26 +88,33 @@ public class FacesUtils
         return (HttpServletRequest) fc.getExternalContext().getRequest();
     }
     
-    public static String getRequestUserAgent()
+    public static String getRequestUri(FacesContext context)
     {
-        String agent = null;
-        Object req = FacesContext.getCurrentInstance().getExternalContext().getRequest();
-        if (req instanceof HttpServletRequest)
-        {
-            agent = ((HttpServletRequest)req).getHeader("user-agent");
+        ExternalContext ec = context.getExternalContext();
+        Object req = (ec!=null ? ec.getRequest() : null);
+        if (req instanceof HttpServletRequest) {
+            HttpServletRequest hr = (HttpServletRequest)req;
+            return hr.getRequestURI(); 
         }
-        return (agent!=null ? agent : "");
+        return "{No HttpServletRequest}";
     }
     
-    public static boolean isRequestUserAgentIE()
+    public static String getRequestUserAgent(FacesContext context)
     {
-        String userAgent = getRequestUserAgent();
-        return (userAgent.indexOf("MSIE")>=0);
+        ExternalContext ec = context.getExternalContext();
+        Object req = (ec!=null ? ec.getRequest() : null);
+        if (req instanceof HttpServletRequest)
+        {   // get User-Agent
+            String agent = ((HttpServletRequest)req).getHeader("user-agent");
+            return agent;
+        }
+        return "{No HttpServletRequest}";
     }
 
     public static String getRequestContextPath()
     {
-        return FacesContext.getCurrentInstance().getExternalContext().getRequestContextPath();
+        ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
+        return (ec!=null ? ec.getRequestContextPath() : null);
     }
 
     public static Object getRequestAttribute(final String key)
@@ -119,6 +128,12 @@ public class FacesUtils
         return getHttpRequest(fc).getAttribute(key);
     }
 
+    public static <T> T getRequestAttribute(final FacesContext fc, final String key, Class<T> clazz)
+    {
+        Object requestValue = getRequestAttribute(fc, key);
+        return (requestValue != null ? ObjectUtils.convert(clazz, requestValue) : null);
+    }
+
     public static void setRequestAttribute(final String key, Object value)
     {
         FacesContext fc = getContext();
@@ -157,6 +172,12 @@ public class FacesUtils
         return isSkipInputValidation(fc); 
     }
 
+    public static <T> T getActionEventAttribute(ActionEvent ae, String name, Class<T> clazz)
+    {
+        Object item = ae.getComponent().getAttributes().get(name);
+        return ObjectUtils.convert(clazz, item);
+    }
+
     /* Navigation */
     
     public static void redirectDirectly(final FacesContext fc, final String url)
@@ -387,6 +408,13 @@ public class FacesUtils
         return getMessage(getContext(), messageKey);
     }
 
+    public static String formatMessage(String msgKey, Object... params)
+    {
+        TextResolver tr = getTextResolver(FacesContext.getCurrentInstance());
+        String pattern = tr.resolveKey(msgKey);
+        return MessageFormat.format(pattern, params);
+    }
+
     /*
     public static void addInfoMessage(FacesContext fc, String clientId, String msg)
     {
@@ -460,7 +488,7 @@ public class FacesUtils
         // redirect to target page
         FacesMessage facesMsg = getFacesErrorMessage(fc, t);
         ExternalContext ec = fc.getExternalContext();
-        ec.getSessionMap().put("PAGE_SESSION_MESSAGE", facesMsg);
+        ec.getSessionMap().put(Page.SESSION_MESSAGE, facesMsg);
         // redirect
         FacesUtils.redirectDirectly(fc, pageTarget);
     }
