@@ -26,14 +26,14 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.ResourceBundle;
 
-import javax.faces.FactoryFinder;
 import javax.faces.application.Application;
-import javax.faces.application.ApplicationFactory;
 import javax.faces.application.FacesMessage;
 import javax.faces.application.FacesMessage.Severity;
+import javax.faces.application.ViewExpiredException;
 import javax.faces.component.NamingContainer;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIViewRoot;
+import javax.faces.context.ExternalContext;
 import javax.faces.context.FacesContext;
 import javax.servlet.ServletContext;
 import javax.servlet.http.HttpServletRequest;
@@ -44,6 +44,7 @@ import org.apache.empire.data.DataType;
 import org.apache.empire.db.DBDatabase;
 import org.apache.empire.db.context.DBRollbackManager;
 import org.apache.empire.db.context.DBRollbackManager.ReleaseAction;
+import org.apache.empire.db.exceptions.UserLevelException;
 import org.apache.empire.exceptions.InternalException;
 import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.NotSupportedException;
@@ -124,11 +125,12 @@ public abstract class WebApplication
         servletContext.setAttribute("app", this);
         // Init
         init(servletContext);
-        // text resolvers
-        log.info("*** initTextResolvers() ***");
-        ApplicationFactory appFactory = (ApplicationFactory) FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
-        Application app = appFactory.getApplication();
-        initTextResolvers(app);
+        // post init
+        if (this.textResolvers==null)
+        {   // text resolvers
+            log.info("*** initTextResolvers() ***");
+            initTextResolvers(startupContext.getApplication());
+        }
         // Log info
         log.info("*** WebApplication initialization complete ***");
         log.info("JSF-Implementation is '{}'", facesImpl.getClass().getName());
@@ -269,6 +271,48 @@ public abstract class WebApplication
         return false;
     }
 
+    /**
+     * Logs an exception as Error with additional information 
+     * @param message the message to log
+     * @param e the underlying exception
+     */
+    public void logException(Logger logger, String message, Throwable e)
+    {
+        if (logger==null)
+            logger=log;
+        // View Exceptions
+        if ((e instanceof ViewExpiredException)) // || (e instanceof ViewNotFoundException)) -- MyFaces only
+        {   // Session expired
+            logger.warn(message+": "+e.getMessage());
+            return;
+        }
+        // Get Session Info
+        String addlInfo;
+        try {
+            FacesContext fc = FacesContext.getCurrentInstance();
+            ExternalContext ec = (fc!=null ? fc.getExternalContext() : null);
+            String sessionInfo = (ec!=null ? getExceptionSessionInfo(ec.getSessionMap()) : "[NoSession]");
+            addlInfo = StringUtils.coalesce(sessionInfo, "[NoSessionInfo]");
+        } catch(Exception ex) {
+            addlInfo = "[Error:"+ex.getMessage()+"] ";
+        }
+        // Log now
+        if (e instanceof UserLevelException)
+            logger.info(StringUtils.concat(addlInfo, e.getClass().getSimpleName(), ": ", e.getMessage()));
+        else
+            logger.error(StringUtils.concat(addlInfo, message), e);                
+    }
+    
+    /**
+     * Override this to provide additonal session info for an exception
+     * @param sessionMap
+     * @return the additional session info
+     */
+    protected String getExceptionSessionInfo(Map<String, Object> sessionMap)
+    {
+        return null;
+    }
+    
     /**
      * returns true if a form input element has been partially submitted
      * @param fc the Faces Context
@@ -289,10 +333,8 @@ public abstract class WebApplication
      */
     public String getPartialSubmitComponentId(final FacesContext fc)
     {
-        // Override for your JSF component Framework. e.g. for IceFaces
-        // Map<String,String> parameterMap = fc.getExternalContext().getRequestParameterMap();
-        // return parameterMap.get("ice.event.captured");
-        return null;
+        Map<String,String> parameterMap = fc.getExternalContext().getRequestParameterMap();
+        return parameterMap.get("javax.faces.source");  // or javax.faces.partial.execute ?
     }
 
     /**
@@ -420,7 +462,7 @@ public abstract class WebApplication
      * @param db the database for which to obtain a connection
      * @return the connection
      */
-    protected synchronized Connection getConnection(DBDatabase db)
+    protected Connection getConnection(DBDatabase db)
     {
         // Get From Pool
         try
@@ -443,7 +485,7 @@ public abstract class WebApplication
      * @param commit flag whether to commit changes
      * @param dbrm the rollback manager
      */
-    protected synchronized void releaseConnection(Connection conn, boolean commit, DBRollbackManager dbrm)
+    protected void releaseConnection(Connection conn, boolean commit, DBRollbackManager dbrm)
     {
         try
         {   // check
