@@ -43,48 +43,53 @@ import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.ItemNotFoundException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
 import org.apache.empire.jsf2.app.TextResolver;
+import org.apache.empire.jsf2.utils.TagStyleClass;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 public abstract class InputControl
 {
 
-    private static final Logger log                   = LoggerFactory.getLogger(InputControl.class);
-    
+    private static final Logger log                    = LoggerFactory.getLogger(InputControl.class);
+
+    // StyleClass 
+    public static final String  CSS_STYLE_CLASS        = "styleClass";
+
     // format attributes
-    public static final String  FORMAT_NULL                   = "null:";
-    public static final String  FORMAT_NULL_ATTRIBUTE         = "format:null";
+    public static final String  FORMAT_NULL            = "null:";
+    public static final String  FORMAT_NULL_ATTRIBUTE  = "format:null";
+    public static final String  FORMAT_NO_VALUE_STYLES = "noValueStyles";
+    /* obsolte from 2024-06-03
     public static final String  FORMAT_VALUE_STYLES           = "valueStyles";
     public static final String  FORMAT_VALUE_STYLES_ATTRIBUTE = "format:valueStyles";
-    
-    // HTML-TAGS
-    public static final String  HTML_TAG_DIV          = "div"; 
-    public static final String  HTML_TAG_SPAN         = "span";
-    public static final String  HTML_TAG_TABLE        = "table";
-    public static final String  HTML_TAG_TR           = "tr";
-    public static final String  HTML_TAG_TD           = "td";
-    public static final String  HTML_TAG_INPUT        = "input";
-    public static final String  HTML_TAG_LABEL        = "label";
-    
-    // HTML-ATTRIBUTES
-    public static final String  HTML_ATTR_ID          = "id";
-    public static final String  HTML_ATTR_CLASS       = "class";
-    public static final String  HTML_ATTR_STYLE       = "style";
-    public static final String  HTML_ATTR_TYPE        = "type";
-    public static final String  HTML_ATTR_DISABLED    = "disabled";
-    public static final String  HTML_ATTR_CHECKED     = "checked";
-    
-    // HTML
-    public static String HTML_EXPR_NBSP = "&nbsp;";
-    
-    // style classes
-    public static final String STYLECLASS_REQUIRED    = " eInpReq";
-    public static final String STYLECLASS_MODIFIED    = " eInpModified";
+    */
 
+    // HTML-TAGS
+    public static final String  HTML_TAG_DIV           = "div";
+    public static final String  HTML_TAG_SPAN          = "span";
+    public static final String  HTML_TAG_TABLE         = "table";
+    public static final String  HTML_TAG_TR            = "tr";
+    public static final String  HTML_TAG_TD            = "td";
+    public static final String  HTML_TAG_INPUT         = "input";
+    public static final String  HTML_TAG_LABEL         = "label";
+
+    // HTML-ATTRIBUTES
+    public static final String  HTML_ATTR_ID           = "id";
+    public static final String  HTML_ATTR_CLASS        = "class";
+    public static final String  HTML_ATTR_STYLE        = "style";
+    public static final String  HTML_ATTR_TYPE         = "type";
+    public static final String  HTML_ATTR_DISABLED     = "disabled";
+    public static final String  HTML_ATTR_CHECKED      = "checked";
+
+    // HTML
+    public static String        HTML_EXPR_NBSP         = "&nbsp;";
+
+    /*
     public InputControl()
     {
         InputControl.log.info("InputControl of class {} created.", getClass().getName());
     }
+    */
 
     /**
      * DisabledType
@@ -169,17 +174,24 @@ public abstract class InputControl
         Object getAttributeEx(String name); /* check Column attributes too, and resolves references to other columns. */
     }
 
-    private String name;
+    private final String name;
+    private final String cssStyleClass;
 
     protected InputControl(String name)
     {
         this.name = name;
+        this.cssStyleClass = initCssStyleClass();
     }
 
     public final String getName()
     {
         return this.name;
     }
+    
+    public final String getCssStyleClass() 
+    {
+        return this.cssStyleClass;
+    }
 
     public String getLabelForId(InputInfo ii)
     {
@@ -237,7 +249,7 @@ public abstract class InputControl
      * @param tooltip the title of the HTML wrapper tag (optional)
      * @param vi the value info
      * @param context the FacesContext
-     * @throws IOException
+     * @throws IOException from ResponseWriter
      */
     public void renderValue(UIComponent comp, String tagName, String styleClass, String tooltip, ValueInfo vi, FacesContext context)
         throws IOException
@@ -245,11 +257,12 @@ public abstract class InputControl
         // writer
         ResponseWriter writer = context.getResponseWriter();
         // has tag?
+        Object value = vi.getValue(true);
         if (tagName!=null)
         {   // write start tag
             writer.startElement(tagName, comp);
-            if (hasFormatOption(vi, FORMAT_VALUE_STYLES, FORMAT_VALUE_STYLES_ATTRIBUTE))
-                styleClass = addDataValueStyle(vi, vi.getValue(true), styleClass);
+            if (!hasFormatOption(vi, FORMAT_NO_VALUE_STYLES))
+                styleClass = addDataValueStyle(vi, value, styleClass);
             if (StringUtils.isNotEmpty(styleClass))
                 writer.writeAttribute("class", styleClass, null);
             if (StringUtils.isNotEmpty(tooltip))
@@ -260,7 +273,7 @@ public abstract class InputControl
                 writer.writeAttribute("style", style, null);
         }
         // render Value
-        renderValue(vi, writer);
+        renderValue(value, vi, writer);
         // has tag?
         if (tagName!=null)
         {   // write end tag
@@ -272,12 +285,12 @@ public abstract class InputControl
      * Renders the control value without a surrounding tag (Text only)
      * @param vi the value info
      * @param writer the output writer
-     * @throws IOException
+     * @throws IOException from ResponseWriter
      */
-    public void renderValue(ValueInfo vi, ResponseWriter writer)
+    public void renderValue(Object value, ValueInfo vi, ResponseWriter writer)
         throws IOException
     {
-        String text = formatValue(vi);
+        String text = formatValue(value, vi);
         writer.append((StringUtils.isEmpty(text) ? HTML_EXPR_NBSP : text));
     }
 
@@ -286,7 +299,7 @@ public abstract class InputControl
      * @param comp the JSF component
      * @param ii the input info
      * @param context the FacesContext
-     * @throws IOException
+     * @throws IOException from ResponseWriter
      */
     public void renderInput(UIComponent comp, InputInfo ii, FacesContext context)
         throws IOException
@@ -301,12 +314,12 @@ public abstract class InputControl
     
     public void updateInputState(UIComponent parent, InputInfo ii, FacesContext context, PhaseId phaseId)
     {
-        List<UIComponent> cl = parent.getChildren(); 
-        if (cl.isEmpty())
+        List<UIComponent> children = parent.getChildren(); 
+        if (children.isEmpty())
             return;
-        updateInputState(cl, ii, context, phaseId);
+        // update state
+        updateInputState(children, ii, context, phaseId);
         // update attached objects
-        List<UIComponent> children = parent.getChildren();
         while (!(parent instanceof UIInput))
             parent = parent.getParent();
         for (UIComponent child : children)
@@ -397,12 +410,17 @@ public abstract class InputControl
         return submittedValue;
     }
     
+    protected String initCssStyleClass() 
+    {
+        return StringUtils.concat(TagStyleClass.INPUT_TYPE_PREFIX.get(), name.substring(0,1).toUpperCase(), name.substring(1));
+    }
+    
     /**
      * adds style attributes related to the current value
-     * @param vi
+     * @param vi the value info
      * @param value the current value
-     * @param styleClass
-     * @return
+     * @param styleClass the style class
+     * @return the data value string
      */
     protected String addDataValueStyle(ValueInfo vi, Object value, String styleClass)
     {
@@ -499,13 +517,6 @@ public abstract class InputControl
             reqMap.remove(clientId);
     }
 
-    /**
-     * Override this to format a value for output
-     * 
-     * @param value
-     * @param ii
-     * @return
-     */
     protected Object formatInputValue(Object value, InputInfo ii)
     {
         return value;
@@ -601,7 +612,7 @@ public abstract class InputControl
 
     protected void setInputStyleClass(UIInput input, String cssStyleClass)
     {
-        input.getAttributes().put("styleClass", cssStyleClass);
+        input.getAttributes().put(InputControl.CSS_STYLE_CLASS, cssStyleClass);
     }
 
     protected void copyAttributes(UIComponent parent, InputInfo ii, UIInput input, String additonalStyle)
@@ -643,14 +654,11 @@ public abstract class InputControl
         String addlStyles = null;
         if (ii.isRequired())
         {   // required
-            addlStyles = STYLECLASS_REQUIRED;
+            addlStyles = TagStyleClass.INPUT_REQ.get();
         }
         if (ii.isModified()) 
-        {   // modified only
-            if (addlStyles==null)
-                addlStyles = STYLECLASS_MODIFIED;
-            else
-                addlStyles+= STYLECLASS_MODIFIED;
+        {   // modified
+            addlStyles = TagStyleClass.INPUT_MOD.addTo(addlStyles);
         }
         copyAttributes(parent, ii, input, addlStyles);
     }
@@ -669,35 +677,40 @@ public abstract class InputControl
     
     public void addRemoveValueNullStyle(UIInput input, boolean nullValue)
     {
-        addRemoveStyle(input, " eValNull", nullValue);
+        addRemoveStyle(input, TagStyleClass.VALUE_NULL, nullValue);
     }
 
     public void addRemoveDisabledStyle(UIInput input, boolean disabled)
     {
-        addRemoveStyle(input, " eInpDis", disabled);
+        addRemoveStyle(input, TagStyleClass.INPUT_DIS, disabled);
     }
 
     public void addRemoveInvalidStyle(UIInput input, boolean invalid)
     {
-        addRemoveStyle(input, " eInvalid", invalid);
+        addRemoveStyle(input, TagStyleClass.VALUE_INVALID, invalid);
     }
-
+    
     public void addRemoveStyle(UIInput input, String styleName, boolean setStyle)
     {
-        String styleClass = StringUtils.toString(input.getAttributes().get("styleClass"), "");
-        boolean hasStyle = (styleClass.indexOf(styleName) >= 0);
+        String styleClasses = StringUtils.toString(input.getAttributes().get(InputControl.CSS_STYLE_CLASS), "");
+        boolean hasStyle = TagStyleClass.existsIn(styleClasses, styleName);
         if (setStyle == hasStyle)
             return; // Nothing to do
         // Special IceFaces patch
-        if (styleClass.endsWith("-dis"))
-            styleClass = styleClass.substring(0, styleClass.length() - 4);
+        if (styleClasses.endsWith("-dis"))
+            styleClasses = styleClasses.substring(0, styleClasses.length() - 4);
         // add or remove disabled style
         if (setStyle)
-            styleClass += styleName;
+            styleClasses = TagStyleClass.addTo(styleClasses, styleName);
         else
-            styleClass = styleClass.replace(styleName, "");
+            styleClasses = TagStyleClass.removeFrom(styleClasses, styleName);
         // add Style
-        setInputStyleClass(input, styleClass);
+        setInputStyleClass(input, styleClasses);
+    }
+
+    public final void addRemoveStyle(UIInput input, TagStyleClass style, boolean setStyle)
+    {
+        this.addRemoveStyle(input, style.get(), setStyle);
     }
     
     /**
@@ -723,38 +736,60 @@ public abstract class InputControl
             // Error
             InputControl.log.error("The enum '" + ((Enum<?>) value).name() + "' has no text!");
         }
-        // Lookup and Print value
+        // Lookup and return text
         Options options = vi.getOptions();
-        if (options != null && !options.isEmpty() && !hasFormatOption(vi, "nolookup"))
-        {   // Check for Options
-            OptionEntry entry = options.getEntry(value);
-            if (entry!=null)
-                return vi.getText(entry.getText());
-            // Error
-            if (value!=null)
-                InputControl.log.error("The element '" + String.valueOf(value) + "' is not part of the supplied option list.");
+        if (options != null && !hasFormatOption(vi, "nolookup"))
+        {   // getOptionText
+            String text = getOptionText(options, value, vi);
+            if (text!=null)
+                return text;
         }
         // value
         if (value == null)
             value = getFormatOption(vi, InputControl.FORMAT_NULL, InputControl.FORMAT_NULL_ATTRIBUTE);
         // Convert to String
-        String s = StringUtils.toString(value, "");
+        String s = StringUtils.toString(value, StringUtils.EMPTY);
         if (hasFormatOption(vi, "noencode"))
             return s;
         // Encode Html
         return escapeHTML(s);
     }
 
+    /**
+     * Returns the display text for an option
+     * @param options
+     * @param value
+     * @param vi
+     * @return the display text or null if the option value could not be resolved
+     */
+    protected String getOptionText(Options options, Object value, ValueInfo vi)
+    {
+        if (options == null)
+            throw new InvalidArgumentException("options", options);
+        // Check for Options
+        OptionEntry entry = options.getEntry(value);
+        if (entry!=null)
+            return vi.getText(entry.getText());
+        // Check empty
+        if (ObjectUtils.isEmpty(value))
+            return StringUtils.EMPTY;
+        // Error: Value not found! 
+        String column = (vi.getColumn()!=null ? vi.getColumn().getName() : "?");
+        log.error("The element \"{}\" is not part of the supplied option list for column {}", value, column);
+        return null; 
+    }
+
     /**
      * Returns the value formated as a string
      * This is a shortcut for formatString(vi.getValue(), vi)
      * Derived classes may override formatString
-     */
+     
     protected final String formatValue(ValueInfo vi)
     {
         // boolean hasError = ((vi instanceof InputInfo) && !((InputInfo)vi).isValid()); 
         return formatValue(vi.getValue(true), vi);
     }
+    */
 
     /**
      * escapes a String for html
