@@ -60,6 +60,8 @@ public class TextInputControl extends InputControl
     public static final String  DATE_FORMAT           = "date-format:";
     public static final String  DATE_FORMAT_ATTRIBUTE = "format:date";
 
+    public static final String  FRACTION_DIGITS       = "fraction-digits:";
+    
     private Class<? extends HtmlInputText> inputComponentClass;
 
     public TextInputControl(String name, Class<? extends HtmlInputText> inputComponentClass)
@@ -125,7 +127,7 @@ public class TextInputControl extends InputControl
         HtmlInputText input = (HtmlInputText) comp;
         if (ii.isInsideUIData())
         {   // always reset the style class inside UIData
-            String tagStyle = StringUtils.toString(ii.getAttribute("styleClass"), null);
+            String tagStyle = StringUtils.toString(ii.getAttribute(InputControl.CSS_STYLE_CLASS), null);
             String cssStyle = getInputStyleClass(ii, tagStyle);
             input.setStyleClass(cssStyle);
             /*
@@ -168,7 +170,7 @@ public class TextInputControl extends InputControl
         String dataTypeClass = TagEncodingHelper.getDataTypeClass(ii.getColumn().getDataType()); 
         String styleClass    = TagEncodingHelper.getTagStyleClass(tagStyle, dataTypeClass, null, null);
         */
-        span.getAttributes().put("styleClass", tagStyle);
+        span.getAttributes().put(InputControl.CSS_STYLE_CLASS, tagStyle);
         span.getChildren().add(text);
         // done 
         return span;
@@ -259,16 +261,13 @@ public class TextInputControl extends InputControl
 
     protected String formatValue(Object value, ValueInfo vi, boolean escapeHTML)
     {
-        // Lookup and Print value
+        // Lookup and return text
         Options options = vi.getOptions();
-        if (options != null && !options.isEmpty() && !hasFormatOption(vi, "nolookup"))
-        { // Check for Options
-            String text = options.get(value);
-            if (StringUtils.isNotEmpty(text))
-                return vi.getText(text);
-            // Error
-            if (value!=null)
-                TextInputControl.log.error("The element '" + String.valueOf(value) + "' is not part of the supplied option list.");
+        if (options != null && !hasFormatOption(vi, "nolookup"))
+        {   // getOptionText
+            String text = getOptionText(options, value, vi);
+            if (text!=null)
+                return text;
         }
         // Check Value
         if (value == null)
@@ -300,7 +299,7 @@ public class TextInputControl extends InputControl
         }
         if (dataType == DataType.DECIMAL || dataType == DataType.FLOAT)
         { // Dezimal oder Double
-            NumberFormat nf = getNumberFormat(dataType, vi.getLocale(), column);
+            NumberFormat nf = getNumberFormat(dataType, vi, column);
             return nf.format(value);
         }
         if (dataType == DataType.DATE || dataType == DataType.DATETIME || dataType == DataType.TIMESTAMP)
@@ -355,10 +354,10 @@ public class TextInputControl extends InputControl
     // ------- render -------
 
     @Override
-    public void renderValue(ValueInfo vi, ResponseWriter writer)
+    public void renderValue(Object value, ValueInfo vi, ResponseWriter writer)
         throws IOException
     {
-        String text = formatValue(vi);
+        String text = formatValue(value, vi);
         if (StringUtils.isEmpty(text))
         { // nothing
             writer.append(HTML_EXPR_NBSP);
@@ -437,7 +436,6 @@ public class TextInputControl extends InputControl
             if (ml > 0)
                 return ml;
         }
-
         Column col = ii.getColumn();
         // cast to DBTableColumn 
         DataType type = col.getDataType();
@@ -518,28 +516,55 @@ public class TextInputControl extends InputControl
         return DataType.UNKNOWN;
     }
 
-    protected NumberFormat getNumberFormat(DataType dataType, Locale locale, Column column)
+    protected NumberFormat getNumberFormat(DataType dataType, ValueInfo vi, Column column)
     {
+        Locale locale = vi.getLocale();
         if (column == null)
             return NumberFormat.getNumberInstance(locale);
         // Column is supplied
         String type = StringUtils.valueOf(column.getAttribute(Column.COLATTR_NUMBER_TYPE));
-        NumberFormat nf = null;
-        if (type.equalsIgnoreCase("Integer"))
-            nf = NumberFormat.getIntegerInstance(locale);
-        else
-            nf = NumberFormat.getNumberInstance(locale);
+        boolean isInteger = "Integer".equalsIgnoreCase(type);
+        NumberFormat nf = (isInteger) ? NumberFormat.getIntegerInstance(locale)
+                                      : NumberFormat.getNumberInstance(locale); 
         // Groups Separator?
         Object groupSep = column.getAttribute(Column.COLATTR_NUMBER_GROUPSEP);
         nf.setGroupingUsed(groupSep != null && ObjectUtils.getBoolean(groupSep));
         // Fraction Digits?
-        Object fractDigit = column.getAttribute(Column.COLATTR_FRACTION_DIGITS);
-        if (fractDigit != null)
-        {   int fractionDigits = ObjectUtils.getInteger(fractDigit);
-            int length = (int)column.getSize();
-            int fractionColumn = (int)(column.getSize()*10)-(length*10);
-            nf.setMaximumFractionDigits(Math.max(fractionDigits, fractionColumn));
-            nf.setMinimumFractionDigits(fractionDigits);
+        Object limitFractionDigits = (isInteger ? null : getFormatOption(vi, FRACTION_DIGITS, Column.COLATTR_FRACTION_DIGITS));
+        if (limitFractionDigits != null)
+        {   // get column limits
+            int minFactionDigits = 0;
+            int maxFactionDigits = -1;
+            if (!(limitFractionDigits instanceof Number)) {
+                // not a number
+                String limit = limitFractionDigits.toString();
+                if (limit.startsWith("min:")) {
+                    minFactionDigits = ObjectUtils.getInteger(limit.substring(4), 0);
+                }
+                else if (limit.startsWith("max:")) {
+                    maxFactionDigits = ObjectUtils.getInteger(limit.substring(4), -1);
+                }
+                else if ("auto".equals(limitFractionDigits)) {
+                    minFactionDigits = 999; /* make equal to maxFactionDigits */
+                }
+                else if (!"limit".equals(limitFractionDigits)) {
+                    minFactionDigits = maxFactionDigits = ObjectUtils.getInteger(limit, -1);
+                }
+            }
+            else minFactionDigits = maxFactionDigits = ((Number)limitFractionDigits).intValue();
+            // check range
+            if (minFactionDigits<0)
+                minFactionDigits = 0;
+            if (maxFactionDigits<0) {
+                // Detect from column
+                int length = (int)column.getSize();
+                maxFactionDigits = (int)(column.getSize()*10)-(length*10);
+                if (minFactionDigits > maxFactionDigits)
+                    minFactionDigits = maxFactionDigits;
+            }
+            // Set 
+            nf.setMinimumFractionDigits(minFactionDigits);
+            nf.setMaximumFractionDigits(maxFactionDigits);
         }
         // IntegerDigits (left-padding)
         Object intDigits = column.getAttribute(Column.COLATTR_INTEGER_DIGITS);
