@@ -21,13 +21,15 @@ package org.apache.empire.commons;
 import java.io.Serializable;
 import java.util.AbstractSet;
 import java.util.ArrayList;
-import java.util.HashSet;
+import java.util.Collection;
 import java.util.Iterator;
+import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
 
 import org.apache.empire.data.DataType;
 import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.xml.XMLUtil;
 import org.w3c.dom.Element;
 
@@ -37,14 +39,114 @@ import org.w3c.dom.Element;
  * Additionally the class provides a text string describing the value for display purposes.<BR>
  * The class is implemented as a set of OptionEntry objects 
  * where the entry value is used as the key for the set and thus must be unique.<BR>
- * <P> 
  */
 public class Options extends AbstractSet<OptionEntry> implements Cloneable, Serializable
 {
 	private static final long serialVersionUID = 1L;
 
-    private static final String EMPTY_STRING = "";
+    /**
+     * Implements the Map interface for Options
+     */
+    private class ImmutableMap<T> implements Map<T,String>
+    {
+        @Override
+        public int size()
+        {
+            return Options.this.size();
+        }
 
+        @Override
+        public boolean isEmpty()
+        {
+            return Options.this.isEmpty();
+        }
+
+        @Override
+        public boolean containsKey(Object key)
+        {
+            return contains(key);
+        }
+    
+        @Override
+        public boolean containsValue(Object value)
+        {
+            for (OptionEntry e : Options.this.list)
+            {
+                if (ObjectUtils.compareEqual(value, e.getText()))
+                    return true;
+            }
+            return false;
+        }
+
+        @SuppressWarnings("unchecked")
+        @Override
+        public Set<T> keySet()
+        {
+            Set<T> keySet = new ArraySet<T>(this.size());
+            for (OptionEntry e : list)
+                keySet.add((T)e.getValue());
+            return keySet;
+        }
+    
+        @Override
+        public Collection<String> values()
+        {
+            Collection<String> textValues = new ArrayList<String>(this.size());
+            for (OptionEntry e : list)
+                textValues.add(e.getText());
+            return textValues;
+        }
+
+        @SuppressWarnings("unchecked")
+        @Override
+        public Set<Entry<T, String>> entrySet()
+        {
+            ArraySet<Entry<T, String>> set = new ArraySet<Entry<T, String>>(list.size());
+            for (OptionEntry e : list)
+                set.add(new ArrayMap.Entry<T,String>((T)e.getValue(), e.getText()));
+            return set.immutable();
+        }
+
+        @Override
+        public String get(Object key)
+        {
+            return Options.this.get(key);
+        }
+
+        @Override
+        public String put(Object key, String value)
+        {
+            throw new NotSupportedException(this, "put");
+        }
+    
+        @Override
+        public void putAll(Map<? extends T, ? extends String> map)
+        {
+            throw new NotSupportedException(this, "putAll");
+        }
+
+        @Override
+        public String remove(Object key)
+        {
+            throw new NotSupportedException(this, "remove");
+        }
+
+        @Override
+        public void clear()
+        {
+            throw new NotSupportedException(this, "clear");
+        }
+        
+        @Override
+        public String toString()
+        {
+            return Options.this.toString();
+        }
+    }
+    
+    /**
+     * InsertPos enum
+     */
 	public enum InsertPos
     {
         Top, Bottom, Sort
@@ -57,6 +159,11 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         this.list = new ArrayList<OptionEntry>();
     }
     
+    public Options(int initialCapacity)
+    {   // Default constructor
+        this.list = new ArrayList<OptionEntry>(initialCapacity);
+    }
+    
     public Options(Options other)
     {
         this.list = new ArrayList<OptionEntry>(other.size());
@@ -83,10 +190,18 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         for (int i=0; i<items.length; i++)
         {
             Enum<?> item = items[i];
-            add(item, item.toString(), true);
+            append(item, item.toString(), true);
         }
     }
 
+    @Override
+    public boolean addAll(Collection<? extends OptionEntry> source)
+    {
+        for (OptionEntry e : source)
+            add((OptionEntry)e.clone());
+        return true;
+    }
+
     @Override
     public Options clone()
     {
@@ -131,7 +246,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
     public String get(Object value)
     {
         OptionEntry oe = getEntry(value);
-        return (oe!=null ? oe.getText() : EMPTY_STRING);
+        return (oe!=null ? oe.getText() : StringUtils.EMPTY);
     }
 
     public boolean isActive(Object value)
@@ -161,15 +276,57 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      */
     public String getTextAt(int i)
     {
-        return (i>=0 && i<list.size() ? list.get(i).getText() : EMPTY_STRING);
+        return (i>=0 && i<list.size() ? list.get(i).getText() : StringUtils.EMPTY);
     }
 
-    public Set<Object> getValues()
+    /**
+     * Returns all values as a set
+     * boolean activeOnly flag whether to return the active items only
+     * @return the value set
+     */
+    public Set<Object> getValues(boolean activeOnly)
     {
-        HashSet<Object> set = new HashSet<Object>(list.size());
+        int count = (activeOnly ? getActiveCount() : list.size());
+        ArraySet<Object> values = new ArraySet<Object>(count);
         for (OptionEntry e : list)
-            set.add(e.getValue());
-        return set;
+            if (activeOnly==false || e.isActive())
+                values.add(e.getValue());
+        return values;
+    }
+    
+    /**
+     * Returns all values as a set
+     * @return the value set
+     */
+    public final Set<Object> getValues()
+    {
+        return getValues(false);
+    }
+
+    /**
+     * Returns the number of active elements
+     * @return the number of active elements
+     */
+    public int getActiveCount()
+    {
+        int count = 0;
+        for (OptionEntry e : list)
+            if (e.isActive())
+                count++;
+        return count;
+    }
+    
+    /**
+     * Returns the subset of active options 
+     * @return the active options
+     */
+    public Options getActive()
+    {
+        Options o = new Options(getActiveCount());
+        for (OptionEntry e : list)
+            if (e.isActive())
+                o.list.add(e);
+        return o;
     }
 
     /**
@@ -177,21 +334,21 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * 
      * @param value the value object
      * @param text the text
+     * @param active flag if element is active (selectable)
      * @param pos the position, see {@link InsertPos}
      */
-    public void set(Object value, String text, boolean active, InsertPos pos)
+    public void set(Object value, String text, Boolean active, InsertPos pos)
     {
         if (text == null)
-        { // text must not be null!
-            return;
-        }
+            text = StringUtils.EMPTY;
         // Find Entry
         OptionEntry oe = getEntry(value);
         if (oe!=null)
-        {   // already present
+        {   // replace
             oe.setText(text);
-            oe.setActive(active);
-        } 
+            if (active!=null)
+                oe.setActive(active);
+        }
         else
         {   // find insert pos
             int index;
@@ -202,7 +359,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
             else // bottom is default
                 index = list.size();
             // add entry now
-            list.add(index, createOptionEntry(value, text, active));
+            list.add(index, createOptionEntry(value, text, (active!=null ? active :true )));
         }
     }
 
@@ -213,9 +370,9 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * @param text the text
      * @param pos the position, see {@link InsertPos}
      */
-    public void set(Object value, String text, InsertPos pos)
+    public final void set(Object value, String text, InsertPos pos)
     {
-        set(value, text, true, pos);
+        set(value, text, (Boolean)null, pos);
     }
     
     /**
@@ -224,40 +381,66 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * @param value the value object
      * @param text the text
      */
-    public void set(Object value, String text)
+    public void set(Object value, String text, Boolean active)
     {
-        set(value, text, InsertPos.Bottom);
+        if (text == null)
+            text = StringUtils.EMPTY;
+        // check if it exists
+        OptionEntry oe = getEntry(value);
+        if (oe!=null)
+        {   // replace
+            oe.setText(text);
+            if (active!=null)
+                oe.setActive(active);
+        }
+        else
+        {   // add new Option
+            list.add(createOptionEntry(value, text, (active!=null ? active :true )));
+        }
+    }
+    
+    /**
+     * Sets or adds Adds an option at the bottom
+     * 
+     * @param value the value object
+     * @param text the text
+     */
+    public final void set(Object value, String text)
+    {
+        set(value, text, (Boolean)null);
     }
 
     /**
-     * Adds an object, the check for an existing can be skipped for
-     * performance issues (not recommended!)
-     * 
+     * Adds or updates an option
+     * Same as set() but allows Option building 
      * @param value the value
-     * @param text the text
-     * @param noCheck set to true to skip testing for an existing key (handle with care!)
+     * @param text the text for this value
+     * @param active flag if element is active (selectable)
      */
-    public void add(Object value, String text, boolean active)
+    public final Options add(Object value, String text, boolean active)
     {
-        OptionEntry oe = getEntry(value);
-        if (oe!=null)
-        {
-            oe.setText(text);
-            oe.setActive(active);
-        }
-        else
-        {
-            list.add(createOptionEntry(value, text, active));
-        }
+        set(value, text, active);
+        return this;
+    }
+
+    /**
+     * Adds or updates an option
+     * Same as set() but allows Option building 
+     * @param value the value
+     * @param text the text for this value
+     */
+    public final Options add(Object value, String text)
+    {
+        set(value, text, (Boolean)null);
+        return this;
     }
 
     @Override
     public boolean add(OptionEntry option)
     {
-        if (option.getText() == null)
-        { // text must not be null!
-            return false;
-        }
+        if (option==null || option.getText() == null)
+            throw new InvalidArgumentException("option", option);
+        // find and add or replace
         OptionEntry oe = getEntry(option.getValue());
         if (oe!=null)
             list.set(getIndex(oe), option);
@@ -265,6 +448,21 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
             list.add(option);
         return true;
     }
+    
+    /**
+     * Appends an option
+     * Useful for fast loading when it is certain that there are no duplicates
+     * WARNING: Does not check if the entry already exists 
+     * @param value the value object
+     * @param text the text
+     * @param active the flag whether or not this item is active
+     */
+    public void append(Object value, String text, boolean active)
+    {
+        if (text == null)
+            text = StringUtils.EMPTY;
+        list.add(createOptionEntry(value, text, active));
+    }
 
     @Override
     public void clear()
@@ -278,13 +476,19 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         return (getEntry(value)!=null);
     }
 
+    /**
+     * Checks if the Option list contains an empty value
+     * @return true if it contains an empty value or false otherwise
+     */
     public boolean containsNull()
     {   // Check if exits
-        return (getEntry("")!=null);
+        return (getEntry(StringUtils.EMPTY)!=null);
     }
     
     /**
      * same as contains(), but IDE may not issue warning
+     * @param value the value to check
+     * @return true if it exists in the options list
      */
     public boolean has(Object value)
     {   // Check if exits
@@ -326,6 +530,24 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
     {
         return list.toArray();
     }
+    
+    /**
+     * Returns an immutable Map for the options 
+     * @return the map of options
+     */
+    public Map<Object, String> map()
+    {
+        return map(Object.class);
+    }
+    
+    /**
+     * Returns an immutable Map for the options 
+     * @return the map of options
+     */
+    public <T> Map<T, String> map(Class<T> type)
+    {
+        return new ImmutableMap<T>();
+    }
 
     @Override
     public String toString()
@@ -351,7 +573,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * Adds all these options to the xml element
      * 
      * @param element the element to add the option tags to
-     * @param flags not used for now
+     * @param dataType the dataType of the element
      */
     public void addXml(Element element, DataType dataType)
     { 
@@ -370,7 +592,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         }
     }
 
-    private int findInsertPos(String text)
+    protected int findInsertPos(String text)
     {
         int i = 0;
         for (; i < list.size(); i++)
@@ -381,4 +603,5 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         }
         return i;
     }
+
 }
