@@ -39,7 +39,6 @@ import org.w3c.dom.Element;
 /**
  * This abstract class handles the creation of the SQL-Commands.
  * There are inner classes to construct different SQL-Commands.
- * <P>
  */
 public abstract class DBCommandExpr extends DBExpr
 {
@@ -57,13 +56,13 @@ public abstract class DBCommandExpr extends DBExpr
          * @param cmd the command expression
          * @param exprList 
          */
-        public DBCmdQuery(DBCommandExpr cmd, DBColumnExpr[] exprList)
+        public DBCmdQuery(DBCommandExpr cmd, DBDatabase db, DBColumnExpr[] exprList)
         { // Set the column expressions
-            super(cmd.getDatabase());
+            super(db);
             this.cmd = cmd;
             // Add Expressions to list
             for (int i = 0; i < exprList.length; i++)
-                columns.add(exprList[i].getSourceColumn());
+                columns.add(exprList[i].getUpdateColumn());
         }
 
         /** Not applicable - returns null */
@@ -99,15 +98,15 @@ public abstract class DBCommandExpr extends DBExpr
         /**
          * Creates the SQL-Command adds the select statement into the SQL-Command.
          * 
-         * @param buf the SQL-Command
+         * @param sql the SQL-Command
          * @param context the current SQL-Command context
          */
         @Override
-        public void addSQL(StringBuilder buf, long context)
+        public void addSQL(DBSQLBuilder sql, long context)
         {
-            buf.append("(");
-            buf.append(cmd.getSelect());
-            buf.append(")");
+            sql.append("(");
+            sql.append(cmd);
+            sql.append(")");
         }
 
         /**
@@ -174,13 +173,13 @@ public abstract class DBCommandExpr extends DBExpr
         /**
          * create the SQL-Command set the expression name to the SQL-Command
          * 
-         * @param buf the SQL-Command
+         * @param sql the SQL-Command
          * @param context the current SQL-Command context
          */
         @Override
-        public void addSQL(StringBuilder buf, long context)
+        public void addSQL(DBSQLBuilder sql, long context)
         { // append UNQUALIFIED name only!
-            buf.append(expr.getName());
+            sql.append(expr.getName());
         }
 
         /**
@@ -241,10 +240,7 @@ public abstract class DBCommandExpr extends DBExpr
             if (attributes != null && attributes.indexOf(name)>=0)
                 return attributes.get(name);
             // Otherwise ask expression
-            DBColumn column = expr.getSourceColumn();
-            if (column!=null)
-                return column.getAttribute(name);
-            return null;
+            return expr.getAttribute(name);
         }
 
         /**
@@ -256,10 +252,7 @@ public abstract class DBCommandExpr extends DBExpr
             if (options != null)
                 return options;
             // Otherwise ask expression
-            DBColumn column = expr.getSourceColumn();
-            if (column!=null)
-                return column.getOptions();
-            return null;
+            return expr.getOptions();
         }
 
         /**
@@ -283,13 +276,27 @@ public abstract class DBCommandExpr extends DBExpr
     }
 
     // Members
+    private final DBMSHandler     dbms;
     protected DBCmdQuery          cmdQuery = null;
     protected List<DBOrderByExpr> orderBy  = null;
 
-    /** Constructs an empty DBCommandExpr object */
-    public DBCommandExpr()
+    /**
+     * Constructs an empty DBCommandExpr object 
+     * @param dbms the database handler
+     */
+    public DBCommandExpr(DBMSHandler dbms)
     {
         // Default Constructor
+        this.dbms = dbms;
+    }
+    
+    /**
+     * returns the DBMSHandler this command belongs to
+     * @return the DBMSHandler
+     */
+    public final DBMSHandler getDbms()
+    {
+        return dbms;
     }
 
     /**
@@ -320,7 +327,8 @@ public abstract class DBCommandExpr extends DBExpr
     public abstract boolean hasSelectExpr();
 
     /**
-     * returns whether or not the command has a specific select expression 
+     * returns whether or not the command has a specific select expression
+     * @param expr the column expr 
      * @return true if the command contains the given select expression of false otherwise
      */
     public abstract boolean hasSelectExpr(DBColumnExpr expr);
@@ -328,19 +336,21 @@ public abstract class DBCommandExpr extends DBExpr
     /**
      * Returns the list of all select expressions as an array
      * Used internally only
+     * @return the select expression array
      */
     public abstract DBColumnExpr[] getSelectExprList();
     
     /**
      * returns a list of expressions for the SELECT part of the sql statement
+     * @return the select expression list
      */
     public abstract List<DBColumnExpr> getSelectExpressions();
 
     /**
      * returns an SQL select command
-     * @param buf the string builder to add the command to
+     * @param sql the sql builder to add the command to
      */
-    public abstract void getSelect(StringBuilder buf);
+    public abstract void getSelect(DBSQLBuilder sql);
 
     /**
      * returns an SQL select command for querying records.
@@ -348,14 +358,16 @@ public abstract class DBCommandExpr extends DBExpr
      */
     public final String getSelect()
     {
-        StringBuilder sql = new StringBuilder();
+        DBSQLBuilder sql = createSQLBuilder(null);
         getSelect(sql);
         return sql.toString();
     }
     
+    public abstract DBCmdParams getParams();
+
     /**
      * returns an array holding all parameter values in the order of their occurrence.
-     * To ensure the correct order, getSelect() should be called first.
+     * To ensure the correct order, getSelect() must be called first.
      * @return an array of command parameter values 
      */
     public abstract Object[] getParamValues();
@@ -383,15 +395,20 @@ public abstract class DBCommandExpr extends DBExpr
     /**
      * Creates the SQL-Command.
      * 
-     * @param buf the SQL-Command
+     * @param sql the SQL-Command
      * @param context the current SQL-Command context
      */
     @Override
-    public void addSQL(StringBuilder buf, long context)
+    public void addSQL(DBSQLBuilder sql, long context)
     {
-        buf.append("(");
-        buf.append(getSelect());
-        buf.append(")");
+        // parenthesis open
+        if ((context & CTX_NOPARENTHESIS)==0)
+            sql.append("(");
+        // append commaand
+        sql.append(this);
+        // parenthesis close
+        if ((context & CTX_NOPARENTHESIS)==0)
+            sql.append(")");
     }
 
     /**
@@ -404,7 +421,6 @@ public abstract class DBCommandExpr extends DBExpr
      */
     public DBCommandExpr union(DBCommandExpr other)
     {   // give dbms a chance to subclass DBCombinedCmd
-    	DBMSHandler dbms = getDatabase().getDbms();
     	return dbms.createCombinedCommand(this, "UNION", other);
     }
 
@@ -418,7 +434,6 @@ public abstract class DBCommandExpr extends DBExpr
      */
     public DBCommandExpr unionAll(DBCommandExpr other)
     {   // give dbms a chance to subclass DBCombinedCmd
-        DBMSHandler dbms = getDatabase().getDbms();
         return dbms.createCombinedCommand(this, "UNION ALL", other);
     }
 
@@ -431,12 +446,12 @@ public abstract class DBCommandExpr extends DBExpr
      */
     public DBCommandExpr intersect(DBCommandExpr other)
     {   // give dbms a chance to subclass DBCombinedCmd
-    	DBMSHandler dbms = getDatabase().getDbms();
     	return dbms.createCombinedCommand(this, "INTERSECT", other);
     }
     
     /**
      * Returns whether or not the command has order by set
+     * @return flag whether an order by is provided
      */
     public boolean hasOrderBy()
     {
@@ -523,7 +538,9 @@ public abstract class DBCommandExpr extends DBExpr
     /**
      * set the maximum number of rows to return when executing a query command
      * A negative value will remove the limit.
-     *
+     * 
+     * @param numRows the number of rows to limit
+     * @return itself (this)
      */
     public DBCommandExpr limitRows(int numRows)
     {
@@ -533,6 +550,9 @@ public abstract class DBCommandExpr extends DBExpr
     /**
      * sets the offset of the first row to return when executing a query command.
      * A negative value will remove the offset.
+     * 
+     * @param numRows the number of rows to skip
+     * @return itself (this)
      */
     public DBCommandExpr skipRows(int numRows)
     {
@@ -591,7 +611,22 @@ public abstract class DBCommandExpr extends DBExpr
     }
 
     /**
-     * returns column expression that is specific for to this command and detached from its source.
+     * creates a new DBSQLBuilder 
+     * @param initalSQL
+     * @return the new DBSQLBuilder
+     */
+    protected DBSQLBuilder createSQLBuilder(String initalSQL)
+    {
+        DBSQLBuilder sql = dbms.createSQLBuilder();
+        if (initalSQL!=null)
+            sql.append(initalSQL);
+        return sql;
+    }
+    
+    /**
+     * wraps a column expression such that is specific for to this command and detached from its source.
+     * @param col the column expression to wrap
+     * @return column expression that is specific for to this command
      */
     protected DBColumnExpr getCmdColumn(DBColumnExpr col)
     {
@@ -606,32 +641,36 @@ public abstract class DBCommandExpr extends DBExpr
         }
         // Check if we already have a Command Query
         if (cmdQuery == null)
-            cmdQuery = new DBCmdQuery(this, getSelectExprList());
+            cmdQuery = new DBCmdQuery(this, col.getDatabase(), getSelectExprList());
         // create a command column
         return new DBCmdColumn(cmdQuery, col);
     }
 
     /**
      * Internally used to build a string from a list of database expressions
-     * @param buf the sql target buffer
-     * @param list list of database objects
+     * @param sql the sql target buffer
+     * @param list the list of expressions to add to the sql
      * @param context the sql command context
      * @param separator string to use as separator between list items
      */
-    protected void addListExpr(StringBuilder buf, List<? extends DBExpr> list, long context, String separator)
+    protected void addListExpr(DBSQLBuilder sql, List<? extends DBExpr> list, long context, String separator)
     {
         for (int i = 0; i < list.size(); i++)
         {   // assemble select columns
             if (i > 0)
-                buf.append(separator);
-            list.get(i).addSQL(buf, context);
+                sql.append(separator);
+            // append
+            list.get(i).addSQL(sql, context);
         }
     }
-
+    
     /**
      * Create the insert into SQL-Command which copies data
      * from a select statement to a destination table.
      * 
+     * @param table the table to insert 
+     * @param select the selected columns
+     * @param columns the list of column expressions
      * @return the insert into SQL-Command
      */
     protected String getInsertInto(DBTable table, DBColumnExpr[] select, List<DBColumnExpr> columns)
@@ -639,8 +678,8 @@ public abstract class DBCommandExpr extends DBExpr
         if (select == null)
             throw new ObjectNotValidException(this);
         // prepare buffer
-        StringBuilder buf = new StringBuilder("INSERT INTO ");
-        table.addSQL(buf, CTX_FULLNAME);
+        DBSQLBuilder sql = createSQLBuilder("INSERT INTO ");
+        table.addSQL(sql, CTX_FULLNAME);
         // destination columns
         if (columns != null && columns.size() > 0)
         { // Check Count
@@ -649,15 +688,15 @@ public abstract class DBCommandExpr extends DBExpr
                 throw new InvalidArgumentException("columns", "size()!=select.length");
             }
             // Append Names
-            buf.append(" (");
-            addListExpr(buf, columns, CTX_NAME, ", ");
-            buf.append(")");
+            sql.append(" (");
+            addListExpr(sql, columns, CTX_NAME, ", ");
+            sql.append(")");
         }
         // append select statement
-        buf.append("\r\n");
-        getSelect(buf);
+        sql.append("\r\n");
+        getSelect(sql);
         // done
-        return buf.toString();
+        return sql.toString();
     }
     
 }
