@@ -77,9 +77,6 @@ public abstract class DBRecordData extends DBObject
     public abstract int addXmlData(Element parent);
     public abstract Document getXmlDocument();
 
-    // others
-    public abstract void close();
-
     /**
      * Returns a value based on an index.
      */
@@ -87,7 +84,7 @@ public abstract class DBRecordData extends DBObject
     public abstract Object getValue(int index);
     
     /**
-     * @Deprecated Renamed to get(...)   
+     * Deprecated Renamed to get(...)   
      */
     @Deprecated
     public Object getValue(ColumnExpr column)
@@ -96,7 +93,7 @@ public abstract class DBRecordData extends DBObject
     }
 
     /**
-     * @Deprecated Renamed to get(...)   
+     * Deprecated Renamed to get(...)   
      */
     @Deprecated
     public final <T> T getValue(Column column, Class<T> returnType)
@@ -105,7 +102,7 @@ public abstract class DBRecordData extends DBObject
     }
 
     /**
-     * @Deprecated Renamed to get(...)   
+     * Deprecated Renamed to get(...)   
      */
     @Deprecated
     public final Object[] getValues(ColumnExpr... columns)
@@ -142,7 +139,7 @@ public abstract class DBRecordData extends DBObject
     /**
      * Returns an array of values for the given column expressions
      * 
-     * @param column the column expressions
+     * @param columns the column expressions
      * @return the corresponding record values
      */
     public final Object[] getArray(ColumnExpr... columns)
@@ -397,7 +394,7 @@ public abstract class DBRecordData extends DBObject
             // Illegal value
             String value = StringUtils.valueOf(getValue(index));
             log.error("Unable to resolve enum value of '{}' for type {}", value, enumType.getName());
-            throw new FieldIllegalValueException(col.getSourceColumn(), value, e);
+            throw new FieldIllegalValueException(col.getUpdateColumn(), value, e);
         }
     }
 
