@@ -62,7 +62,7 @@ import org.apache.empire.exceptions.ItemNotFoundException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.ObjectNotValidException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
-import org.apache.empire.exceptions.UnspecifiedErrorException;
+import org.apache.empire.exceptions.InvalidOperationException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -425,10 +425,11 @@ public abstract class DBRowSet extends DBExpr implements EntityType
      */
     public int getColumnIndex(ColumnExpr columnExpr)
     {
+        columnExpr = ObjectUtils.unwrap(columnExpr);
         if (columnExpr instanceof DBColumn)
             return getColumnIndex((DBColumn)columnExpr);
         else {
-            Column source = columnExpr.getSourceColumn();
+            Column source = columnExpr.getUpdateColumn();
             if (source instanceof DBColumn)
                 return getColumnIndex((DBColumn)source);
         }
@@ -524,7 +525,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
         if (timestampColumn!=null && timestampColumn.getRowSet()!=this)
             throw new InvalidArgumentException("timestampColumn", timestampColumn);
         if (timestampColumn!=null && this.timestampColumn!=null && this.timestampColumn!=timestampColumn)
-            throw new UnspecifiedErrorException("A Timestamp column has already been set for rowset "+getName());
+            throw new InvalidOperationException("A Timestamp column has already been set for rowset "+getName());
         if (timestampColumn instanceof DBTableColumn)
             ((DBTableColumn) timestampColumn).setReadOnly(true);
         // set now
@@ -821,7 +822,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
                     i++;
                 }
             }
-            log.warn("Record [{}] not found in {}", StringUtils.toString(key, cmd.getSelect()), getName());
+            log.warn("Record [{}] not found in {}", StringUtils.toString(key), getName());
             // throw RecordNotFoundException
             throw new RecordNotFoundException(this, key);
         } finally {
