@@ -22,6 +22,7 @@ import java.sql.Timestamp;
 import java.text.SimpleDateFormat;
 import java.time.LocalDate;
 import java.time.LocalDateTime;
+import java.time.LocalTime;
 import java.util.Collection;
 import java.util.Date;
 
@@ -218,6 +219,17 @@ public abstract class DBSQLBuilder implements Appendable
             appendSimpleValue(dataType, value);
         }
     }
+
+    /**
+     * Appends the SQL representation of a value
+     * @param dataType the DataType
+     * @param value an DBExpr object, array or a basis data type(e.g. int, String)
+     * @param context the context of the DBColumnExpr object
+     */
+    public final void appendValue(DataType dataType, Object value, long context)
+    {
+        appendValue(dataType, value, context, getPhrase(DBSqlPhrase.SQL_CONCAT_EXPR));
+    }
     
     /**
      * Expands an SQL template and adds it to the SQL command
@@ -375,6 +387,10 @@ public abstract class DBSQLBuilder implements Appendable
         {   // Convert LocalDateTime to Timestamp
             ts = java.sql.Timestamp.valueOf((LocalDateTime)value);
         }
+        else if ((value instanceof LocalTime))
+        {   // Convert LocalTime to Timestamp with current date
+            ts = java.sql.Timestamp.valueOf(((LocalTime)value).atDate(LocalDate.now()));
+        }
         else 
         {   // "Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]"
             String dtValue = value.toString().trim();
