@@ -51,7 +51,6 @@ import org.apache.empire.db.list.DataBean;
 import org.apache.empire.dbms.DBMSFeature;
 import org.apache.empire.dbms.DBMSHandler;
 import org.apache.empire.exceptions.InternalException;
-import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -495,7 +494,7 @@ public class DBUtils implements DBContextAware
             return querySingleInt(sql, subCmd.getParamValues(), 0);
         }
         // find any rowset
-        DBRowSet rs = exprList[0].getSourceColumn().getRowSet();
+        DBRowSet rs = exprList[0].getRowSet();
         // create the count command
         DBCommand countCmd = cmd.clone();
         countCmd.clearSelect();
@@ -607,6 +606,9 @@ public class DBUtils implements DBContextAware
     /**
      * Fills an option list provided with the result from a query.
      * The option list is filled with the values of the first and second column.
+     * Optionally a third column may provide a boolean value for active or disabled.
+     *
+     * IMPORTANT: The query must contain unique values in the first column!
      * 
      * @param sqlCmd the sql command
      * @param sqlParams the command params
@@ -625,15 +627,15 @@ public class DBUtils implements DBContextAware
             rs = dbms.executeQuery(sqlCmd, sqlParams, false, context.getConnection());
             if (rs == null)
                 throw new UnexpectedReturnValueException(rs, "dbms.executeQuery()");
-            if (rs.getMetaData().getColumnCount()<2)
-                throw new InvalidArgumentException("sqlCmd", sqlCmd);
-            // Check Result
+            // Load options
+            int colCount = rs.getMetaData().getColumnCount();
             int count = 0;
             while (rs.next())
             {
                 Object value = rs.getObject(1);
-                String text  = rs.getString(2);
-                options.add(value, text, true);
+                String text  = rs.getString((colCount>=2) ? 2 : 1);
+                boolean active = (colCount>=3) ? ObjectUtils.getBoolean(rs.getObject(3)) : true;
+                options.append(value, text, active);
                 count++;
             }
             // Debug
