@@ -21,6 +21,7 @@ package org.apache.empire.db.expr.compare;
 import java.util.Set;
 
 import org.apache.empire.commons.ObjectUtils;
+import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.DataType;
 import org.apache.empire.db.DBCmdParam;
 import org.apache.empire.db.DBCmpType;
@@ -30,8 +31,10 @@ import org.apache.empire.db.DBCommand;
 import org.apache.empire.db.DBCommandExpr;
 import org.apache.empire.db.DBDatabase;
 import org.apache.empire.db.DBExpr;
+import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.DBSQLBuilder;
 import org.apache.empire.db.expr.column.DBAliasExpr;
+import org.apache.empire.dbms.DBSqlPhrase;
 
 
 /**
@@ -83,6 +86,15 @@ public class DBCompareColExpr extends DBCompareExpr
     {
         return expr.getDatabase();
     }
+    
+    /**
+     * Returns the underlying rowset containing this column
+     */
+    @Override
+    public DBRowSet getRowSet()
+    {
+        return expr.getRowSet();
+    }
 
     /** 
      * Gets the DBColumnExpr object 
@@ -201,7 +213,7 @@ public class DBCompareColExpr extends DBCompareExpr
                 arraySep = ", ";
                 break;
             default:
-                arraySep = "+";
+                arraySep = (expr.getDataType().isNumeric() ? "+" : sql.getPhrase(DBSqlPhrase.SQL_CONCAT_EXPR));
                 break;
         }
         // Add comparison operator and value
@@ -293,6 +305,21 @@ public class DBCompareColExpr extends DBCompareExpr
         // Add Comparison Value
         addCompareExpr(sql, context);
     }
+    
+    /**
+     * For Debugging
+     */
+    @Override
+    public String toString()
+    {
+        StringBuilder b = new StringBuilder(expr.toString());
+        b.append(cmpop.toString());
+        if (value instanceof DBCmdParam)
+            b.append("?");
+        else
+            b.append(StringUtils.toString(value, StringUtils.NULL));
+        return b.toString();
+    }
 
     /**
      * Returns whether the constraint should replace another one or not.
@@ -321,5 +348,23 @@ public class DBCompareColExpr extends DBCompareExpr
     	// other types
     	return equals(other);
     }
+
+    /**
+     * Returns whether the constraint is on the given column
+     * 
+     * @return true it the constraint is on the given column or false otherwise
+     */
+    @Override
+    public boolean isConstraintOn(DBColumnExpr colExpr)
+    {
+        // compare columns
+        if (ObjectUtils.compareEqual(expr, colExpr))
+            return true;
+        // Update column
+        if ((colExpr instanceof DBColumn) && !(expr instanceof DBColumn) && colExpr.equals(expr.getUpdateColumn()))
+            return true;
+        // not found
+        return false;
+    }
     
 }
\ No newline at end of file
