@@ -21,9 +21,11 @@ package org.apache.empire.db.expr.compare;
 import java.util.Set;
 
 import org.apache.empire.db.DBColumn;
+import org.apache.empire.db.DBColumnExpr;
 import org.apache.empire.db.DBCommand;
 import org.apache.empire.db.DBCommandExpr;
 import org.apache.empire.db.DBDatabase;
+import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.DBSQLBuilder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -76,6 +78,15 @@ public class DBExistsExpr extends DBCompareExpr
     {
         return cmd.getDatabase();
     }
+    
+    /**
+     * Returns the underlying rowset containing this column
+     */
+    @Override
+    public DBRowSet getRowSet()
+    {
+        return (compareExpr!=null ? compareExpr.getRowSet() : null);
+    }
 
     /**
      * Prepare function
@@ -165,5 +176,15 @@ public class DBExistsExpr extends DBCompareExpr
     	}
     	return false;
     }
-    
+
+    /**
+     * Returns whether the constraint is on the given column
+     * @return true it the constraint is on the given column or false otherwise
+     */
+    @Override
+    public boolean isConstraintOn(DBColumnExpr colExpr)
+    {
+        return (compareExpr!=null ? compareExpr.isConstraintOn(colExpr) : false);
+    }
+   
 }
\ No newline at end of file
